diff -pruN 1.2.12-1/bootstrap 1.2.16-1/bootstrap
--- 1.2.12-1/bootstrap	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/bootstrap	2020-11-30 18:21:59.000000000 +0000
@@ -28,15 +28,11 @@ display_help () {
 }
 
 # Check for parameters
-if [ "${1}x" = "--helpx" -o "${1}x" = "-hx" ]; then
+if [ "${1}" = "--help" -o "${1}" = "-h" ]; then
   display_help
   exit 0
 fi
 
-# Check for dos2unix
-which dos2unix > /dev/null 2>&1
-[ $? -gt 0 ] && echo "FATAL: Unable to locate dos2unix utility" && exit -1
-
 echo "INFO: Starting Spine build process"
 
 # Remove software build specific directories
@@ -44,7 +40,10 @@ echo "INFO: Removing cache directories"
 rm -rf autom4te.cache .deps
 
 # Make sure all files are unix formatted files 
-find . -type f -exec dos2unix --d2u \{\} \; > /dev/null 2>&1
+which dos2unix > /dev/null 2>&1
+if [ $? -eq 0 ]; then
+  find . -type f -exec dos2unix --d2u \{\} \; > /dev/null 2>&1
+fi
 
 # Prepare a build state
 echo "INFO: Running auto-tools to verify buildability"
diff -pruN 1.2.12-1/CHANGELOG 1.2.16-1/CHANGELOG
--- 1.2.12-1/CHANGELOG	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/CHANGELOG	2020-11-30 18:21:59.000000000 +0000
@@ -1,5 +1,23 @@
 The Cacti Group | spine
 
+1.2.16
+-issue: Some developer debug log messages falsely labeled as WARNINGS
+-issue#164: Remove the need of the dos2unix program
+-issue#171: Spine experiencing MySQL socket error 2002 under load
+-issue#175: Under heavy load MySQL/MariaDB return 2006 and 2013 errors on query
+-feature: Add backtrace output to stderr for signals
+-feature#176: Add Data Source turnaround time to debug output
+
+1.2.15
+-issue#166: Special characters may not always be ignored properly
+-issue#168: Correct issues with linking under OpenBSD
+
+1.2.14
+-issue#162: Spine not updating rrd_next_step
+
+1.2.13
+-issue#156: Compile on Cygwin is failing due to icmp6 headers missing
+
 1.2.12
 -issue#155: Failed host lookup causes spine to crash
 
diff -pruN 1.2.12-1/common.h 1.2.16-1/common.h
--- 1.2.12-1/common.h	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/common.h	2020-11-30 18:21:59.000000000 +0000
@@ -104,7 +104,9 @@
 #  include <netinet/in.h>
 #  include <netinet/ip.h>
 #  include <netinet/ip6.h>
+#ifndef __CYGWIN__
 #  include <netinet/icmp6.h>
+#endif
 #  include <netinet/ip_icmp.h>
 #endif
 
diff -pruN 1.2.12-1/configure.ac 1.2.16-1/configure.ac
--- 1.2.12-1/configure.ac	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/configure.ac	2020-11-30 18:21:59.000000000 +0000
@@ -1,5 +1,5 @@
 AC_PREREQ(2.53)
-AC_INIT(Spine Poller, 1.2.12, http://www.cacti.net/issues.php)
+AC_INIT(Spine Poller, 1.2.16, http://www.cacti.net/issues.php)
 
 AC_CONFIG_AUX_DIR(config)
 AC_SUBST(ac_aux_dir)
@@ -8,6 +8,7 @@ AC_CANONICAL_HOST
 AC_CONFIG_SRCDIR(spine.c)
 AC_PREFIX_DEFAULT(/usr/local/spine)
 AC_LANG(C)
+AC_PROG_CC
 
 AM_INIT_AUTOMAKE([foreign])
 AC_CONFIG_HEADERS(config/config.h)
@@ -256,12 +257,22 @@ if test -n "$MYSQL_LIB_DIR" ; then
 fi
   CFLAGS="-I$MYSQL_INC_DIR $CFLAGS"
 
-AC_CHECK_LIB(mysqlclient, mysql_init,
-  [ LIBS="-lmysqlclient -lm -ldl $LIBS"
-    AC_DEFINE(HAVE_MYSQL, 1, MySQL Client API)
-    HAVE_MYSQL=yes ],
-  [ HAVE_MYSQL=no ]
-)
+unamestr=$(uname)
+if test $unamestr == 'OpenBSD'; then
+  AC_CHECK_LIB(mysqlclient, mysql_init,
+    [ LIBS="-lmysqlclient -lm $LIBS"
+      AC_DEFINE(HAVE_MYSQL, 1, MySQL Client API)
+      HAVE_MYSQL=yes ],
+    [ HAVE_MYSQL=no ]
+  )
+else
+  AC_CHECK_LIB(mysqlclient, mysql_init,
+    [ LIBS="-lmysqlclient -lm -ldl $LIBS"
+      AC_DEFINE(HAVE_MYSQL, 1, MySQL Client API)
+      HAVE_MYSQL=yes ],
+    [ HAVE_MYSQL=no ]
+  )
+fi
 
 if test $MYSQL_REENTRANT = 1 ; then
   LIBS="-lmysqlclient_r -lm -ldl $LIBS"
@@ -273,7 +284,11 @@ else
       LIBS="-lmysqlclient_r -lm -ldl $LIBS"
     else
       if test "$HAVE_MYSQL" == "yes"; then
-        LIBS="-lmysqlclient -lm -ldl $LIBS"
+        if test $unamestr == 'OpenBSD'; then
+          LIBS="-lmysqlclient -lm $LIBS"
+        else
+          LIBS="-lmysqlclient -lm -ldl $LIBS"
+        fi
       else
         LIBS="-lmariadbclient -lm -ldl $LIBS"
       fi
diff -pruN 1.2.12-1/debian/changelog 1.2.16-1/debian/changelog
--- 1.2.12-1/debian/changelog	2020-05-25 18:46:59.000000000 +0000
+++ 1.2.16-1/debian/changelog	2021-01-03 21:14:02.000000000 +0000
@@ -1,3 +1,9 @@
+cacti-spine (1.2.16-1) unstable; urgency=medium
+
+  * New upstream version 1.2.16
+
+ -- Paul Gevers <elbrus@debian.org>  Sun, 03 Jan 2021 22:14:02 +0100
+
 cacti-spine (1.2.12-1) unstable; urgency=medium
 
   * New upstream version 1.2.12
diff -pruN 1.2.12-1/error.c 1.2.16-1/error.c
--- 1.2.12-1/error.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/error.c	2020-11-30 18:21:59.000000000 +0000
@@ -54,30 +54,67 @@ static void spine_signal_handler(int spi
 	set.exit_code = spine_signal;
 
 	switch (spine_signal) {
+		case SIGABRT:
+			spine_print_backtrace();
+			die("FATAL: Spine Interrupted by Abort Signal");
+			break;
 		case SIGINT:
+			spine_print_backtrace();
 			die("FATAL: Spine Interrupted by Console Operator");
 			break;
 		case SIGSEGV:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered a Segmentation Fault");
 			break;
 		case SIGBUS:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered a Bus Error");
 			break;
 		case SIGFPE:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered a Floating Point Exception");
 			break;
 		case SIGQUIT:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered a Keyboard Quit Command");
 			break;
 		case SIGPIPE:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered a Broken Pipe");
 			break;
 		default:
+			spine_print_backtrace();
 			die("FATAL: Spine Encountered An Unhandled Exception Signal Number: '%d'", spine_signal);
 			break;
 	}
 }
 
+void spine_print_backtrace() {
+	int nptrs;
+	int bt_buf_size = 100;
+	void *buffer[bt_buf_size];
+	char **strings;
+	int j;
+
+	nptrs = backtrace(buffer, bt_buf_size);
+	fprintf(stderr, "backtrace() returned %d addresses\n", nptrs);
+
+	/* The call backtrace_symbols_fd(buffer, nptrs, STDOUT_FILENO)
+	 * would produce similar output to the following: */
+
+	strings = backtrace_symbols(buffer, nptrs);
+	if (strings == NULL) {
+		perror("Spine backtrace symbols follow");
+		exit(EXIT_FAILURE);
+	}
+
+	for (j = 0; j < nptrs; j++) {
+		fprintf(stderr, "%s\n", strings[j]);
+	}
+
+	free(strings);
+}
+
 static int spine_fatal_signals[] = {
 	SIGINT,
 	SIGPIPE,
diff -pruN 1.2.12-1/error.h 1.2.16-1/error.h
--- 1.2.12-1/error.h	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/error.h	2020-11-30 18:21:59.000000000 +0000
@@ -31,5 +31,6 @@
  +-------------------------------------------------------------------------+
 */
 
+extern void spine_print_backtrace(void);
 extern void install_spine_signal_handler(void);
 extern void uninstall_spine_signal_handler(void);
diff -pruN 1.2.12-1/poller.c 1.2.16-1/poller.c
--- 1.2.12-1/poller.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/poller.c	2020-11-30 18:21:59.000000000 +0000
@@ -175,6 +175,8 @@ void poll_host(int host_id, int host_thr
 	char last_snmp_context[65];
 	char last_snmp_engine_id[30];
 	double poll_time = get_time_as_double();
+	double thread_start = 0;
+	double thread_end = 0;
 
 	/* reindex shortcuts to speed polling */
 	int previous_assert_failure = FALSE;
@@ -1090,6 +1092,8 @@ void poll_host(int host_id, int host_thr
 
 		i = 0; k = 0;
 		while ((i < num_rows) && (!host->ignore_host)) {
+			thread_start = get_time_as_double();
+
 			switch(poller_items[i].action) {
 			case POLLER_ACTION_SNMP: /* raw SNMP poll */
 				/* initialize or reinitialize snmp as required */
@@ -1196,10 +1200,12 @@ void poll_host(int host_id, int host_thr
 
 							snprintf(poller_items[snmp_oids[j].array_position].result, RESULTS_BUFFER, "%s", snmp_oids[j].result);
 
+							thread_end = get_time_as_double();
+
 							if (is_debug_device(host_id)) {
-								SPINE_LOG(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+								SPINE_LOG(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 							} else {
-								SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+								SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 							}
 						}
 
@@ -1291,10 +1297,12 @@ void poll_host(int host_id, int host_thr
 
 						snprintf(poller_items[snmp_oids[j].array_position].result, RESULTS_BUFFER, "%s", snmp_oids[j].result);
 
+						thread_end = get_time_as_double();
+
 						if (is_debug_device(host_id)) {
-							SPINE_LOG(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+							SPINE_LOG(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 						} else {
-							SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+							SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 						}
 
 						if (!IS_UNDEFINED(poller_items[snmp_oids[j].array_position].result)) {
@@ -1357,10 +1365,12 @@ void poll_host(int host_id, int host_thr
 
 				if (poll_result) free(poll_result);
 
+				thread_end = get_time_as_double();
+
 				if (is_debug_device(host_id)) {
-					SPINE_LOG(("Device[%i] HT[%i] DS[%i] SCRIPT: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, poller_items[i].arg1, poller_items[i].result));
+					SPINE_LOG(("Device[%i] HT[%i] DS[%i] TT[%.2f] SCRIPT: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, (thread_end - thread_start) * 1000, poller_items[i].arg1, poller_items[i].result));
 				} else {
-					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SCRIPT: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, poller_items[i].arg1, poller_items[i].result));
+					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] TT[%.2f] SCRIPT: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, (thread_end - thread_start) * 1000, poller_items[i].arg1, poller_items[i].result));
 				}
 
 				if (!IS_UNDEFINED(poller_items[i].result)) {
@@ -1413,10 +1423,12 @@ void poll_host(int host_id, int host_thr
 
 				if (poll_result) free(poll_result);
 
+				thread_end = get_time_as_double();
+
 				if (is_debug_device(host_id)) {
-					SPINE_LOG(("Device[%i] HT[%i] DS[%i] SS[%i] SERVER: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, php_process, poller_items[i].arg1, poller_items[i].result));
+					SPINE_LOG(("Device[%i] HT[%i] DS[%i] SS[%i] TT[%.2f] SERVER: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, php_process, (thread_end - thread_start) * 1000, poller_items[i].arg1, poller_items[i].result));
 				} else {
-					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SS[%i] SERVER: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, php_process, poller_items[i].arg1, poller_items[i].result));
+					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SS[%i] TT[%.2f] SERVER: %s, output: %s", host_id, host_thread, poller_items[i].local_data_id, php_process, (thread_end - thread_start) * 1000, poller_items[i].arg1, poller_items[i].result));
 				}
 
 				if (IS_UNDEFINED(poller_items[i].result)) {
@@ -1497,10 +1509,12 @@ void poll_host(int host_id, int host_thr
 
 				snprintf(poller_items[snmp_oids[j].array_position].result, RESULTS_BUFFER, "%s", snmp_oids[j].result);
 
+				thread_end = get_time_as_double();
+
 				if (is_debug_device(host_id)) {
-					SPINE_LOG(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+					SPINE_LOG(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 				} else {
-					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
+					SPINE_LOG_MEDIUM(("Device[%i] HT[%i] DS[%i] TT[%.2f] SNMP: v%i: %s, dsname: %s, oid: %s, value: %s", host_id, host_thread, poller_items[snmp_oids[j].array_position].local_data_id, (thread_end - thread_start) * 1000, host->snmp_version, host->hostname, poller_items[snmp_oids[j].array_position].rrd_name, poller_items[snmp_oids[j].array_position].arg1, poller_items[snmp_oids[j].array_position].result));
 				}
 
 				if (!IS_UNDEFINED(poller_items[snmp_oids[j].array_position].result)) {
diff -pruN 1.2.12-1/README.md 1.2.16-1/README.md
--- 1.2.12-1/README.md	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/README.md	2020-11-30 18:21:59.000000000 +0000
@@ -61,6 +61,7 @@ chmod +s /usr/local/spine/bin/spine
    * gcc-core
    * gzip
    * help2man
+   * inetutils-src
    * libmysqlclient
    * libmariadb-devel
    * libssl-devel
diff -pruN 1.2.12-1/snmp.c 1.2.16-1/snmp.c
--- 1.2.12-1/snmp.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/snmp.c	2020-11-30 18:21:59.000000000 +0000
@@ -186,7 +186,6 @@ void *snmp_host_init(int host_id, char *
 	snprintf(hostnameport, BUFSIZE, "%s:%i", hostname, snmp_port);
 	session.peername    = hostnameport;
 	session.retries     = set.snmp_retries;
-	session.remote_port = snmp_port;
 	session.timeout     = (snmp_timeout * 1000); /* net-snmp likes microseconds */
 
 	SPINE_LOG_MEDIUM(("Device[%i] INFO: SNMP Device '%s' has timeout %ld (%d), retries %d", host_id, hostname, session.timeout, snmp_timeout, session.retries));
@@ -345,27 +344,27 @@ char *snmp_get(host_t *current_host, cha
 	if (current_host->snmp_session != NULL) {
 		anOID_len = MAX_OID_LEN;
 
-		SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_pdu_create(%s)", current_host->id, snmp_oid));
+		SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_pdu_create(%s)", current_host->id, snmp_oid));
 		pdu = snmp_pdu_create(SNMP_MSG_GET);
-		SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_pdu_create(%s) [complete]", current_host->id, snmp_oid));
+		SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_pdu_create(%s) [complete]", current_host->id, snmp_oid));
 
-		SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_parse_oid(%s)", current_host->id, snmp_oid));
+		SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_parse_oid(%s)", current_host->id, snmp_oid));
 		if (!snmp_parse_oid(snmp_oid, anOID, &anOID_len)) {
-			SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_parse_oid(%s) [complete]", current_host->id, snmp_oid));
+			SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_parse_oid(%s) [complete]", current_host->id, snmp_oid));
 			SPINE_LOG(("Device[%i] ERROR: SNMP Get Problems parsing SNMP OID %s", current_host->id, snmp_oid));
 			SET_UNDEFINED(result_string);
 			return result_string;
 		} else {
-			SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_parse_oid(%s) [complete]", current_host->id, snmp_oid));
-			SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_add_null_var(%s)", current_host->id, snmp_oid));
+			SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_parse_oid(%s) [complete]", current_host->id, snmp_oid));
+			SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_add_null_var(%s)", current_host->id, snmp_oid));
 			snmp_add_null_var(pdu, anOID, anOID_len);
-			SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_add_null_var(%s) [complete]", current_host->id, snmp_oid));
+			SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_add_null_var(%s) [complete]", current_host->id, snmp_oid));
 		}
 
 		/* poll host */
-		SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_sess_sync_response(%s)", current_host->id, snmp_oid));
+		SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_sess_sync_response(%s)", current_host->id, snmp_oid));
 		status = snmp_sess_synch_response(current_host->snmp_session, pdu, &response);
-		SPINE_LOG_DEVDBG(("Device[%i] WARNING: snmp_sess_sync_response(%s) [complete]", current_host->id, snmp_oid));
+		SPINE_LOG_DEVDBG(("Device[%i] DEBUG: snmp_sess_sync_response(%s) [complete]", current_host->id, snmp_oid));
 
 		/* add status to host structure */
 		current_host->snmp_status = status;
diff -pruN 1.2.12-1/spine.c 1.2.16-1/spine.c
--- 1.2.12-1/spine.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/spine.c	2020-11-30 18:21:59.000000000 +0000
@@ -695,7 +695,7 @@ int main(int argc, char *argv[]) {
 		total_items = atoi(mysql_row[0]);
 		db_free_result(tresult);
 
-		if (total_items < device_threads) {
+		if (total_items && total_items < device_threads) {
 			device_threads = total_items;
 		}
 
diff -pruN 1.2.12-1/sql.c 1.2.16-1/sql.c
--- 1.2.12-1/sql.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/sql.c	2020-11-30 18:21:59.000000000 +0000
@@ -130,34 +130,32 @@ MYSQL_RES *db_query(MYSQL *mysql, int ty
 			if (error == 2013 && errno == EINTR) {
 				usleep(50000);
 				continue;
-			}
-
-			if ((error == 1213) || (error == 1205)) {
-				#ifndef SOLAR_THREAD
+			} else if (error == 2013 || (error == 2006 && errno == EINTR)) {
+				db_disconnect(mysql);
 				usleep(50000);
-				#endif
+				db_connect(type, mysql);
 				error_count++;
 
 				if (error_count > 30) {
-					SPINE_LOG(("FATAL: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'", query_frag));
+					SPINE_LOG(("FATAL: Too many Reconnect Attempts!"));
 					exit(1);
 				}
 
 				continue;
-			} else if (error == 2006 && errno == EINTR) {
-				db_disconnect(mysql);
+			}
+
+			if ((error == 1213) || (error == 1205)) {
 				usleep(50000);
-				db_connect(type, mysql);
 				error_count++;
 
 				if (error_count > 30) {
-					SPINE_LOG(("FATAL: Too many Reconnect Attempts!"));
+					SPINE_LOG(("FATAL: Too many Lock/Deadlock errors occurred!, SQL Fragment:'%s'", query_frag));
 					exit(1);
 				}
 
 				continue;
 			} else {
-				SPINE_LOG(("FATAL: MySQL Error:'%i', Message:'%s'", error, mysql_error(mysql)));
+				SPINE_LOG(("FATAL: Database Error:'%i', Message:'%s'", error, mysql_error(mysql)));
 				exit(1);
 			}
 		} else {
@@ -236,7 +234,7 @@ void db_connect(int type, MYSQL *mysql)
 	mysql_init(mysql);
 
 	if (mysql == NULL) {
-		SPINE_LOG(("FATAL: MySQL unable to allocate memory and therefore can not connect"));
+		SPINE_LOG(("FATAL: Database unable to allocate memory and therefore can not connect"));
 		exit(1);
 	}
 
@@ -291,7 +289,7 @@ void db_connect(int type, MYSQL *mysql)
 			error = mysql_errno(mysql);
 			db_disconnect(mysql);
 
-			if ((error == 2003 || error == 2013) && errno == EINTR) {
+			if ((error == 2002 || error == 2003 || error == 2006 || error == 2013) && errno == EINTR) {
 				usleep(50000);
 				tries++;
 				success = FALSE;
@@ -299,7 +297,7 @@ void db_connect(int type, MYSQL *mysql)
 			}
 
 			if (error != 1049 && error != 2005 && error != 1045) {
-				printf("MYSQL: Connection Failed: Error:'%u', Message:'%s'\n", mysql_errno(mysql), mysql_error(mysql));
+				printf("Database: Connection Failed: Error:'%u', Message:'%s'\n", mysql_errno(mysql), mysql_error(mysql));
 
 				success = FALSE;
 
diff -pruN 1.2.12-1/util.c 1.2.16-1/util.c
--- 1.2.12-1/util.c	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/util.c	2020-11-30 18:21:59.000000000 +0000
@@ -1171,7 +1171,7 @@ int spine_log(const char *format, ...) {
 
 	/* append a line feed to the log message if needed */
 	if (!strstr(flogmessage, "\n")) {
-		strncat(flogmessage, "\n", 1);
+		strcat(flogmessage, "\n");
 	}
 
 	if ((IS_LOGGING_TO_FILE() &&
@@ -1345,32 +1345,44 @@ int is_numeric(char *string) {
  *
  *  \return TRUE if the string is valid hex, FALSE otherwise
  *
+ *  The function is modified where the string needs to include
+ *  at least one of the following string ' ', '-', or ':'
+ *
  */
-int is_hexadecimal(const char * str, const short ignore_space) {
+int is_hexadecimal(const char * str, const short ignore_special) {
 	int i = 0;
+	int delim_found = FALSE;
 
 	if (!str) return FALSE;
 
 	while (*str) {
 		switch (*str) {
-		case '0': case '1': case '2': case '3':
-		case '4': case '5': case '6': case '7':
-		case '8': case '9':
-		case 'a': case 'A': case 'b': case 'B':
-		case 'c': case 'C': case 'd': case 'D':
-		case 'e': case 'E': case 'f': case 'F':
-		case '"':
-			break;
-		case ' ': case '\t':
-			if (ignore_space) break;
-		default:
-			return FALSE;
+			case '0': case '1': case '2': case '3':
+			case '4': case '5': case '6': case '7':
+			case '8': case '9':
+			case 'a': case 'A': case 'b': case 'B':
+			case 'c': case 'C': case 'd': case 'D':
+			case 'e': case 'E': case 'f': case 'F':
+			case '"':
+				break;
+			case '-': case ':': case ' ':
+				delim_found = TRUE;
+				break;
+			case '\t':
+				if (ignore_special) {
+					break;
+				}
+			default:
+				return FALSE;
 		}
+
 		str++;
 		i++;
 	}
 
-	if (i < 3) return FALSE;
+	if ((i < 3) || delim_found == FALSE) {
+		return FALSE;
+	}
 
 	return TRUE;
 }
diff -pruN 1.2.12-1/util.h 1.2.16-1/util.h
--- 1.2.12-1/util.h	2020-05-03 20:20:09.000000000 +0000
+++ 1.2.16-1/util.h	2020-11-30 18:21:59.000000000 +0000
@@ -51,7 +51,7 @@ extern void set_option(const char *setti
 extern int is_numeric(char *string);
 extern int is_ipaddress(const char *string);
 extern int all_digits(const char *str);
-extern int is_hexadecimal(const char * str, const short ignore_space);
+extern int is_hexadecimal(const char * str, const short ignore_special);
 
 /* determine if a device is a debug device */
 extern int is_debug_device(int device_id);
